@@ -1,22 +1,23 @@
 package Language::Expr::Compiler::Perl;
 BEGIN {
-  $Language::Expr::Compiler::Perl::VERSION = '0.07';
+  $Language::Expr::Compiler::Perl::VERSION = '0.08';
 }
 # ABSTRACT: Compile Language::Expr expression to Perl
 
 use Any::Moose;
 with 'Language::Expr::EvaluatorRole';
-extends 'Language::Expr::Evaluator';
+extends 'Language::Expr::Compiler::Base';
 
-use UUID::Tiny ':std';
 use boolean;
 
 
-has todo => (is => 'rw', default => sub { [] });
+sub rule_pair_simple {
+    my ($self, %args) = @_;
+    my $match = $args{match};
+    "$match->{key} => $match->{value}";
+}
 
-
-
-sub rule_pair {
+sub rule_pair_string {
     my ($self, %args) = @_;
     my $match = $args{match};
     "$match->{key} => $match->{value}";
@@ -210,7 +211,12 @@ sub rule_power {
     join "", grep {defined} @res;
 }
 
-sub rule_subscripting {
+sub rule_subscripting_var {
+    my ($self, %args) = @_;
+    $self->rule_subscripting_expr(%args);
+}
+
+sub rule_subscripting_expr {
     my ($self, %args) = @_;
     my $match = $args{match};
     my $opd = $match->{operand};
@@ -244,11 +250,11 @@ sub rule_undef {
 }
 
 sub rule_squotestr {
-    __quote(Language::Expr::Interpreter::Default::rule_squotestr(@_));
+    $_[0]->_quote(Language::Expr::Interpreter::Default::rule_squotestr(@_));
 }
 
 sub rule_dquotestr {
-    __quote(Language::Expr::Interpreter::Default::rule_dquotestr(@_));
+    $_[0]->_quote(Language::Expr::Interpreter::Default::rule_dquotestr(@_));
 }
 
 sub rule_bool {
@@ -275,32 +281,36 @@ sub rule_func {
     my ($self, %args) = @_;
     my $match = $args{match};
     my $f = $match->{func_name};
+    my $fmap = $self->func_mapping->{$f};
+    $f = $fmap if $fmap;
     my $args = $match->{args};
     "$f(".join(", ", @$args).")";
 }
 
 sub _map_grep_usort {
-    my ($which, $self, %args) = @_;
+    my ($self, $which, %args) = @_;
     my $match = $args{match};
     my $ary = $match->{array};
     my $expr = $match->{expr};
 
     my $perlop = $which eq 'map' ? 'map' : $which eq 'grep' ? 'grep' : 'sort';
-    my $todoid = __uuidgen(); # yes, this is not proper
-    push @{ $self->todo }, [$todoid, $expr];
-    "[$perlop({ TODO-$todoid } \@{$ary})]";
+    my $uuid = $self->new_marker('subexpr', $expr);
+    "[$perlop({ TODO-$uuid } \@{$ary})]";
 }
 
 sub rule_func_map {
-    _map_grep_usort('map', @_);
+    my ($self, %args) = @_;
+    $self->_map_grep_usort('map', %args);
 }
 
 sub rule_func_grep {
-    _map_grep_usort('grep', @_);
+    my ($self, %args) = @_;
+    $self->_map_grep_usort('grep', %args);
 }
 
 sub rule_func_usort {
-    _map_grep_usort('usort', @_);
+    my ($self, %args) = @_;
+    $self->_map_grep_usort('usort', %args);
 }
 
 sub rule_parenthesis {
@@ -319,9 +329,10 @@ sub expr_postprocess {
 
 # can't use regex here (perl segfaults), at least in 5.10.1, because
 # we are in one big re::gr regex.
-sub __quote {
+sub _quote {
+    my ($self, $str) = @_;
     my @c;
-    for my $c (split '', $_[0]) {
+    for my $c (split '', $str) {
         my $o = ord($c);
         if    ($c eq '"') { push @c, '\\"' }
         elsif ($c eq "\\") { push @c, "\\\\" }
@@ -334,26 +345,25 @@ sub __quote {
     '"' . join("", @c) . '"';
 }
 
-sub __uuidgen {
-    UUID::Tiny::create_uuid_as_string(UUID_V4);
-}
-
 
 sub perl {
     my ($self, $expr) = @_;
     my $res = Language::Expr::Parser::parse_expr($expr, $self);
-    for my $todo (@{ $self->todo }) {
-        my $todoid = $todo->[0];
-        my $subexpr = $todo->[1];
+    for my $m (@{ $self->markers }) {
+        my $type = $m->[0];
+        next unless $type eq 'subexpr';
+        my $uuid = $m->[1];
+        my $subexpr = $m->[2];
         my $subres = Language::Expr::Parser::parse_expr($subexpr, $self);
-        $res =~ s/TODO-$todoid/$subres/g;
+        $res =~ s/TODO-$uuid/$subres/g;
     }
-    $self->todo([]);
+    $self->markers([]);
     $res;
 }
 
 sub eval {
     my ($self, $expr) = @_;
+    no strict;
     my $res = eval $self->perl($expr);
     die $@ if $@;
     $res;
@@ -372,7 +382,7 @@ Language::Expr::Compiler::Perl - Compile Language::Expr expression to Perl
 
 =head1 VERSION
 
-version 0.07
+version 0.08
 
 =head1 SYNOPSIS
 
@@ -415,14 +425,12 @@ own variables.
 
 =item * Functions by default simply use Perl functions.
 
+Unless those specified in func_mapping. For example, if
+$compiler->func_mapping->{foo} = "Foo::do_it", then the expression
+'foo(1)' will be compiled into 'Foo::do_it(1)'.
+
 =back
 
-=head1 ATTRIBUTES
-
-=head2 todo => ARRAYREF
-
-Used to remember which subexpression need to be parsed later.
-
 =head1 METHODS
 
 =for Pod::Coverage ^(rule|expr)_.+
